use ndarray::Zip;
use numpy::PyArray2;
use pyo3::prelude::*;

#[pyfunction]
fn update_cells<'a>(py: Python<'a>, cur: &'a PyArray2<u8>) -> PyResult<&'a PyArray2<u8>> {
    let width = cur.shape()[0];
    let height = cur.shape()[1];

    let binding = cur.readonly();
    let cur = binding.as_array();

    let nxt = Zip::indexed(cur).par_map_collect(|(r, c), _| {
        let mut num_alive: u8 = 0;
        for i in -1..2 {
            for j in -1..2 {
                if i == 0 && j == 0 {
                    continue;
                }
                let row = ((r as i16).wrapping_add(i)) as usize;
                let col = ((c as i16).wrapping_add(j)) as usize;
                if row < width && col < height {
                    num_alive += cur[[row, col]];
                }
            }
        }
        if (cur[[r, c]] == 1 && 2 <= num_alive && num_alive <= 3)
            || (cur[[r, c]] == 0 && num_alive == 3)
        {
            1
        } else {
            0
        }
    });

    Ok(PyArray2::from_owned_array(
        py,
        nxt.into_owned().into_dimensionality().unwrap(),
    ))
}

#[pymodule]
fn interop_module_rs(_py: Python, m: &PyModule) -> PyResult<()> {
    m.add_function(wrap_pyfunction!(update_cells, m)?)?;
    Ok(())
}
