import os
from interop_module_rs import update_cells

os.environ["PYGAME_HIDE_SUPPORT_PROMPT"] = '1'

import numpy as np  # noqa: E402
import pygame as pg  # noqa: E402
from numpy.typing import ArrayLike  # noqa: E402

PATH = os.path.dirname(os.path.abspath(__file__))
MAX_FPS = 60  # set to 0 to disable


def draw(surface: pg.Surface,
         cur: ArrayLike,
         sz: int,
         grid: pg.Surface) -> None:
    cur = np.flip(np.rot90(cur*255), 0)
    cur = np.repeat(np.repeat(cur, sz, axis=0), sz, axis=1)
    surf = pg.surfarray.make_surface(cur)
    surface.blit(surf, (0, 0))
    surface.blit(grid, (0, 0))


def init_file(dimx: int,
              dimy: int,
              file: str) -> ArrayLike:
    cells = np.zeros((dimy, dimx)).astype(np.uint8)
    pattern = np.loadtxt(
        os.path.join(PATH, file),
        dtype=np.uint8,
        delimiter=",")
    pos = (3, 3)
    cells[pos[0]:pos[0]+pattern.shape[0],
          pos[1]:pos[1]+pattern.shape[1]] = pattern
    return cells


def random_init(dimx: int,
                dimy: int) -> ArrayLike:
    return np.random.randint(0, 2, (dimy, dimx)).astype(np.uint8)


def create_grid(surface: pg.Surface,
                cellsize: int) -> pg.Surface:
    grid = pg.Surface(surface.get_size())
    for x in range(0, grid.get_width(), cellsize):
        pg.draw.line(grid, (32, 32, 32), (x, 0), (x, grid.get_height()))
    for y in range(0, grid.get_height(), cellsize):
        pg.draw.line(grid, (32, 32, 32), (0, y), (grid.get_width(), y))
    grid.set_colorkey((0, 0, 0))
    return grid


def main(dimx: int,
         dimy: int,
         cellsize: int) -> None:
    pg.init()
    surface = pg.display.set_mode((dimx*cellsize, dimy*cellsize))
    pg.display.set_caption("Game of Life")

    # Middleweight spaceship (MWSS)
    icon = pg.image.load(os.path.join(PATH, "icon.png"))
    pg.display.set_icon(icon)

    # cells = init_file(dimx, dimy, "init_zpr.csv")
    # cells = init_file(dimx, dimy, "init_gun.csv")
    cells = random_init(dimx, dimy)
    grid = create_grid(surface, cellsize)

    paused = False

    clock = pg.time.Clock()
    while True:
        if not paused:
            clock.tick(MAX_FPS)
            pg.display.set_caption(f"Game of Life - {int(clock.get_fps())} FPS")
        else:
            pg.display.set_caption("Game of Life - PAUSED")

        for event in pg.event.get():
            # quit game
            if (event.type == pg.QUIT or
                (event.type == pg.KEYDOWN and
                 event.key in (pg.K_ESCAPE, pg.K_q))):
                pg.quit()
                return

            # pause game
            if (event.type == pg.KEYDOWN and
               event.key in (pg.K_SPACE, pg.K_p)):
                paused = not paused

            # draw cells
            if event.type == pg.MOUSEBUTTONDOWN:
                pos = pg.mouse.get_pos()
                x = pos[0] // cellsize
                y = pos[1] // cellsize
                cells[y, x] = 1 - cells[y, x]

        # draw to screen and update
        draw(surface, cells, cellsize, grid)
        pg.display.update()

        # update grid after drawing so that
        # the user can see the initial state
        if not paused:
            cells = update_cells(cells)


if __name__ == "__main__":
    main(64, 64, 9)
    # main(900, 450, 2)
